#Emulating MongoDB

**Draft** Nothing working yet. Eventually use npm

The goal here is to design two classes of objects: documents and collections. Both are JavaScript object types whose prototypes have MongoDB functionality. 

For just something that uses the query language, check out [query-engine](https://github.com/balupton/query-engine.npm) and crcn's [sift.js](https://github.com/crcn/sift.js) 

The top level (database analog) is a document that consists of arbitrary data and Collections. A Collection is a set of documents. 

Collections can find documents, update them, and create them. 

Documents have methods for setting properties, manipulating arrays, creating collections, and the methods required for a document to answer the collection's question of whether it matches the criteria. 

You start with the top document. 

doc.modify(modifier object, options) 

Modifier object does the modify properties of MongoDB on a document. But it also has the optional key:

$$mod : [ [collection name, selection for document(s), modifier object, options ], ... ]


the modifier object can have a $$mod which can lead to recursing down. 

documents have three own properties:  data, collections, store  along with a _log and a _desc

doc.get(key);  simple retrieval, possible with dot notation 


doc.query(query object, return obj, sort obj) returns true if query is satisfied, false otherwise.


## Collection methods

Let `col` be a **Collection**

col.update( criteria, modifier, options); 


### [Updating](http://www.mongodb.org/display/DOCS/Updating)



update() replaces the document matching criteria entirely with objNew. If you only want to modify some fields, you should use the $ modifiers below.

Here's the MongoDB shell syntax for update():

col.update( criteria, objNew, upsert, multi )

Arguments:

criteria - query which selects the record to update;
objNew - updated object or $ operators (e.g., $inc) which manipulate the object
upsert - if this should be an "upsert" operation; that is, if the record(s) do not exist, insert one. Upsert only inserts a single document.
multi - indicates if all documents matching criteria should be updated rather than just one. Can be useful with the $ operators below.

save() is a convenience method for doing an upsert. 

## [Finding]()

 `col.find( queryObject, selectionObject)`

 Example: `col.find( {x:4, y : 'abd'})` searches all documents in the collection for x being 4 and y being 'abd', returning the whole document.

 Example: `col.find( { $query : { x : 3, y : "abc" }, $orderby : { x : 1 }, $$limit : 10, $$skip : 20, $$maxscan: 100, $$comment: "just a comment" }, {z:true, $$document: true} );`
 
 will find all documents with those x and y values by scanning the first 100, order them by x, skip the first 20, limit them by 10. Comments have no effect but are great for logging. The returned documents will just contain z. The $$ signs are custom extensions though based off the _addSpecial function and/or the chaining. Also the pulling out of a document uses the values true and false for including/excluding the values (use only one type except for _id field which is included by default if it exists). If including, then the true can be replaced by with a string for the new field name in the new object. The $$document is true if a document type should be returned. Other valid answers are false (just a plain object), number, array, string, ....
 
 Inside the $query, the values can also be some code to be evaluated or a regex. The $query is optional. If x and  are floating out there, it is fine. The code checks for $ sign entries and removes them before checking the remainder of the objects. 
 
  Dot Notation and $slice also work. 
  
  You can retrieve partial sub-objects via Dot Notation.

  > t.find({})
  { "_id" : ObjectId("4c23f0486dad1c3a68457d20"), "x" : { "y" : 1, "z" : [ 1, 2, 3 ] } }
  > t.find({}, {'x.y':1})
  { "_id" : ObjectId("4c23f0486dad1c3
  a68457d20"), "x" : { "y" : 1 } }
  

  Retrieving a Subrange of Array Elements
  You can use the $slice operator to retrieve a subrange of elements in an array.

  db.posts.find({}, {comments:{$slice: 5}}) // first 5 comments
  db.posts.find({}, {comments:{$slice: -5}}) // last 5 comments
  db.posts.find({}, {comments:{$slice: [20, 10]}}) // skip 20, limit 10
  db.posts.find({}, {comments:{$slice: [-20, 10]}}) // 20 from end, limit 10
  The examples above will return all fields and only the subset of elements based on the $slice for that field.

  Filtering with $slice does not affect other fields inclusion/exclusion. It only applies within the array being sliced.

  db.posts.find({}, {_id:1, comments:{$slice: 5}}) // first 5 comments, and the _id field only
  
  
  

## Document methods

### Modifier Operations

Instead of replacing the entire document, we can modify a portion of it using modifier objects. See dot notation as well for how to grab an object. 

db.people.update( { name:"Joe" }, { $inc: { n : 1 } } );
The preceding example says, "Find the first document where 'name' is 'Joe' and then increment 'n' by one."

{ $inc : { field : value } }
increments field by the number value if field is present in the object, otherwise sets field to the number value. This can also be used to decrement by using a negative value.

$set

{ $set : { field : value } }
sets field to value. All datatypes are supported with $set.

$unset

{ $unset : { field : 1} }
Deletes a given field. v1.3+

$push

{ $push : { field : value } }
appends value to field, if field is an existing array, otherwise sets field to the array [value] if field is not present. If field is present but is not an array, an error condition is raised.

Multiple arrays may be updated in one operation by comma separating the field : value pairs:

{ $push : { field : value, field2 : value2 } }
$pushAll

{ $pushAll : { field : value_array } }
appends each value in value_array to field, if field is an existing array, otherwise sets field to the array value_array if field is not present. If field is present but is not an array, an error condition is raised.

$addToSet and $each

{ $addToSet : { field : value } }
Adds value to the array only if its not in the array already, if field is an existing array, otherwise sets field to the array value if field is not present. If field is present but is not an array, an error condition is raised.

To add a list of several values to the set use the $each qualifier:

{ $addToSet : { a : { $each : [ 3 , 5 , 6 ] } } }
$pop

{ $pop : { field : 1  } }
removes the last element in an array (ADDED in 1.1)

{ $pop : { field : -1  } }
removes the first element in an array (ADDED in 1.1) |

$pull

{ $pull : { field : _value } }
removes all occurrences of value from field, if field is an array. If field is present but is not an array, an error condition is raised.

In addition to matching an exact value you can also use expressions ($pull is special in this way):

{ $pull : { field : {field2: value} } } removes array elements with field2 matching value 
{ $pull : { field : {$gt: 3} } } removes array elements greater than 3
{ $pull : { field : {<match-criteria>} } } removes array elements meeting match criteria
	 Because of this feature, to use the embedded doc as a match criteria, you cannot do exact matches on array elements.
$pullAll

{ $pullAll : { field : value_array } }
removes all occurrences of each value in value_array from field, if field is an array. If field is present but is not an array, an error condition is raised.

$rename

Version 1.7.2+ only.

{ $rename : { old_field_name : new_field_name } }
Renames the field with name 'old_field_name' to 'new_field_name'. Does not expand arrays to find a match for 'old_field_name'.

For an upsert with modifiers, if no document matches, the criteria is merged with the objNew to generate the final object. 

The $ positional operator
v1.4+

The $ operator (by itself) means "position of the matched array item in the query". Use this to find an array member and then manipulate it. For example:

---
> t.find()
{ "_id" : ObjectId("4b97e62bf1d8c7152c9ccb74"), "title" : "ABC",
  "comments" : [ { "by" : "joe", "votes" : 3 }, { "by" : "jane", "votes" : 7 } ] }

> t.update( {'comments.by':'joe'}, {$inc:{'comments.\$.votes':1}}, false, true )

> t.find()
{ "_id" : ObjectId("4b97e62bf1d8c7152c9ccb74"), "title" : "ABC",
  "comments" : [ { "by" : "joe", "votes" : 4 }, { "by" : "jane", "votes" : 7 } ] }
Currently the $ operator only applies to the first matched item in the query. For example:

> t.find();
{ "_id" : ObjectId("4b9e4a1fc583fa1c76198319"), "x" : [ 1, 2, 3, 2 ] }
> t.update({x: 2}, {$inc: {"x.\$": 1}}, false, true);
> t.find();
{ "_id" : ObjectId("4b9e4a1fc583fa1c76198319"), "x" : [ 1, 3, 3, 2 ] }
---
The positional operator cannot be combined with an upsert since it requires a matching array element. If your update results in an insert then the "$" will literally be used as the field name.


## Dot Notation

This is to reach into subobjects. Using subobjects for matches requires exact equality (exact ordering too in MongoDB). While dot notation allows partial matches. 

## Query Operators

Conditional Operators
<, <=, >, >= $gt, $gte, $lt, $lte  also combined to do range
$all
$exists
$mod
$ne
$in
$nin
$nor
$or  an array of possibilities  
$and
$size
$type

$regex

$$function will take in the key's value as `this`. The return value 

A lot of automag
